/*
 * @(#)HextrisP.h
 *
 * Copyright 1993 - 2008  David A. Bagley, bagleyd@tux.org
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995  Q. Alex Zhao, azhao@cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Private header file for Hextris */

#ifndef _HextrisP_h
#define _HextrisP_h

#include "xwin.h"
#include "timer.h"
#include "polyhex.h"
#include "Hextris.h"

#ifdef WINVER
extern void destroyGame(HBRUSH brush);
extern void resizeGame(HextrisWidget w);
extern void sizeGame(HextrisWidget w);
extern void initializeGame(HextrisWidget w, HBRUSH brush);
extern void exposeGame(HextrisWidget w);
extern void hideGame(HextrisWidget w);
extern void clearGame(HextrisWidget w);
extern void enterGame(HextrisWidget w);
extern void leaveGame(HextrisWidget w);
extern void moveGameInput(HextrisWidget w, int direction);
extern void soundGame(HextrisWidget w);
extern Boolean incrementGameLevel(HextrisWidget w);
extern Boolean decrementGameLevel(HextrisWidget w);
extern Boolean incrementGameUnits(HextrisWidget w);
extern Boolean decrementGameUnits(HextrisWidget w);
extern Boolean incrementGameColumns(HextrisWidget w);
extern Boolean decrementGameColumns(HextrisWidget w);
extern void fixedfillGame(HextrisWidget w);
extern void suddenAppearGame(HextrisWidget w);
extern Boolean cornersGame(HextrisWidget w);
extern Boolean mixedGame(HextrisWidget w);
extern void bonusGame(HextrisWidget w);
extern void cwGame(HextrisWidget w);
extern void plainPieceGame(HextrisWidget w);
extern void falldownGame(HextrisWidget w);
extern void pauseGame(HextrisWidget w);
extern void showNextGame(HextrisWidget w);
extern void gridGame(HextrisWidget w);
extern void fillGame(HextrisWidget w, int n);
extern void setGame(HextrisWidget w, int reason);
extern void setGameNext(HextrisWidget w, int reason,
	int randomNumber, int randomRotation, int randomReflection,
	int units, Boolean corners, Boolean mixed, Boolean newBonus);

#else

/* This gets around C's inability to do inheritance */
typedef struct _HextrisClassPart {
	int         ignore;
} HextrisClassPart;

typedef struct _HextrisClassRec {
	CoreClassPart coreClass;
	HextrisClassPart hextrisClass;
} HextrisClassRec;

extern HextrisClassRec hextrisClassRec;
#endif

#ifndef BUMPSOUND
#define BUMPSOUND "bump"
#endif

#define SYMBOL ':'

#define NUM_FLASHES 4

#define MAX_SIDES 6
#define MAX_ORIENT 2
#define MIN_LEVELS 1
#define MAX_LEVELS 20
#define FG_SHADES 5
#define BG_SHADES 5

#define XPOS(s,y,j,i)   (((y+i+s)&1)?((s%2)*2-1)*(j&1):0)
#define XINC(y,d,i)     (((((y+!i)&1)&&(d!=1))||(((y+i)&1)&&(d==1)))*d)

#define THRESHOLD(x)	((x)*10)

/*** types ***/

typedef enum {FALL, DROP, MOVE_LEFT, MOVE_RIGHT, ROTATE, REFLECT} move_t;

typedef struct {
	int         units;
	Boolean     corners, mixed, bonus;
} Spec;

typedef struct {
	int         units, polyNumber;
	int         xpos, ypos;
	int         size, colorNumber, direction;
	int         bonus, randomRotation, randomReflection;
	long        randomNumber;
} Thing;

typedef struct {
	int         pmid, cid;
} Field;

typedef struct _HextrisPart {
	Thing       curThing, nextThing;
	Field      *field;
	GC          thingGC[MAX_START_POLYHEXES][FG_SHADES];
	Boolean     fixedFill, suddenAppear, cw, plainPiece, showNext, grid;
	Boolean     invert, sound, resetTime, firstFall, paused, gotNew;
	int         level, fill, completedRows;
	int         startLevel, score, newRows;
	Spec        current, next;
	int         units, columns, rows;
	Boolean     corners, mixed, bonus;
	int         delay, numberPolyhexes, pauseStart;
	int         currentPosition, dropPaused;
	Boolean     fall, started, quit, act;
	Boolean     mono, reverse, focus;
	Boolean     scoreOnly, versionOnly;
	int         hexWidth, hexHeight;
	Position    viewWidth, viewHeight;
	Point       offset;
	GC          xorGC;
	GC          inverseGC[BG_SHADES];
#ifdef WINVER
	char        userName[81], scoreFile[81], bumpSound[81];
#else
	int         menu, pixmapSize;
	char       *userName, *scoreFile, *bumpSound;
	Pixel       foreground, background;
	Pixel       thingColor[MAX_START_POLYHEXES];
	XtCallbackList select;
#endif
} HextrisPart;

typedef struct _HextrisRec {
	CorePart    core;
	HextrisPart hextris;
} HextrisRec;

#ifdef WINVER
extern int brighter(const int light);
extern int darker(const int light);
#else
extern Pixel brighter(HextrisWidget w, Pixel pixel);
extern Pixel darker(HextrisWidget w, Pixel pixel);
#endif
extern void fillPolygon(HextrisWidget w, int pm, int color, Point *points,
	Boolean corners, Boolean graphicalXor);
extern void redoNext(HextrisWidget w);
#endif /* _HextrisP_h */
